from traitlets import (
    Unicode, Dict,
)
from ..basewidget import BaseWidget


class MediaItem(BaseWidget):
    """ 
:warning: **Autogenerated class**


    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("MediaItemModel").tag(sync=True)
    
    query = Dict(default_value=None, allow_none=True, help="""If more than one properties used, it means "and".""").tag(sync=True)
    
    option = Dict(default_value=None, allow_none=True, help="""Each item of this array is an echarts option (`ECUnitOption`). It will be applied when this query is matched.""").tag(sync=True)
    
        
