from ipywidgets import (
    widget_serialization
)
from traitlets import (
    Unicode, Instance, List,
)
from ..basewidget import BaseWidget

from .mediaitems.mediaitem import MediaItem

class Media(BaseWidget):
    """ 
:warning: **Autogenerated class**

See [Responsive Mobile-End](tutorial.html#Responsive%20Mobile-End) for details.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("MediaModel").tag(sync=True)
    
    media = List(trait=Instance(MediaItem), default_value=None, allow_none=True).tag(sync=True, **widget_serialization)
    
        
