from traitlets import (
    Unicode, Float,
)
from ..basewidget import BaseWidget


class StateAnimation(BaseWidget):
    """ 
:warning: **Autogenerated class**

Animation configurations of state switchment. Can be configured in each series individually.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("StateAnimationModel").tag(sync=True)
    
    duration = Float(None, allow_none=True, help="""Duration of animation. Animation will be disabled when set to 0.""").tag(sync=True)
    
    easing = Unicode(None, allow_none=True, help="""Easing of animation.""").tag(sync=True)
    
        
